@@ -16,6 +16,7 @@
 #include "ftk/filters/tdgl_vortex_tracker_3d_regular.hh"
 #include "ftk/filters/xgc_blob_filament_tracker.hh"
 #include "ftk/filters/xgc_blob_threshold_tracker.hh"
+#include "ftk/filters/particle_tracer_2d_regular.hh"
 #include "ftk/filters/threshold_tracker.hh"
 #include "ftk/filters/streaming_filter.hh"
 #include "ftk/filters/feature_curve_set_post_processor.hh"
@@ -38,7 +39,7 @@ std::string archived_intersections_filename, // archived_discrete_critical_point
 std::string thread_backend, accelerator;
 std::string type_filter_str;
 int nthreads = std::thread::hardware_concurrency();
-bool affinity = false;
+bool affinity = false, async = false;
 bool verbose = false, timing = false, help = false;
 int nblocks; 
 bool enable_streaming_trajectories = false,
@@ -90,6 +91,7 @@ std::shared_ptr<json_interface> wrapper;
 std::shared_ptr<contour_tracker_regular> tracker_contour;
 std::shared_ptr<tdgl_vortex_tracker_3d_regular> tracker_tdgl;
 std::shared_ptr<critical_line_tracker_3d_regular> tracker_critical_line;
+std::shared_ptr<particle_tracer> tracker_particle;
 std::shared_ptr<threshold_tracker<>> tracker_threshold;
 std::shared_ptr<ndarray_stream<>> stream;
 
@@ -536,6 +538,40 @@ void initialize_xgc_blob_threshold_tracker(diy::mpi::communicator comm)
   tracker->finalize();
 }
 
+void initialize_particle_tracer(diy::mpi::communicator comm)
+{
+  const auto js = stream->get_json();
+  const size_t nd = stream->n_dimensions(),
+               DW = js["dimensions"][0], 
+               DH = js["dimensions"].size() > 1 ? js["dimensions"][1].get<int>() : 0; 
+               // DD = js["dimensions"].size() > 2 ? js["dimensions"][2].get<int>() : 0;
+  const int nt = js["n_timesteps"];
+
+  std::shared_ptr<particle_tracer_2d_regular> t2dr(new particle_tracer_2d_regular(comm));
+  tracker_particle = t2dr;
+  
+  t2dr->set_domain(lattice({0, 0}, {DW-1, DH-1}));
+  t2dr->set_array_domain(lattice({0, 0}, {DW, DH}));
+  t2dr->set_end_timestep(nt - 1);
+  t2dr->set_number_of_threads(nthreads);
+
+  if (comm.rank() == 0) {
+    std::cerr << "input=" << std::setw(2) << stream->get_json() << std::endl;
+  }
+
+  tracker_particle->initialize();
+  tracker_particle->initialize_particles_at_grid_points();
+
+  stream->set_callback([&](int k, const ndarray<double>& field_data) {
+    tracker_particle->push_field_data_snapshot("vector", field_data);
+
+    if (k != 0) tracker_particle->advance_timestep();
+    if (k == stream->n_timesteps() - 1) tracker_particle->update_timestep();
+  });
+  stream->start();
+  stream->finish();
+}
+
 void initialize_critical_line_tracker(diy::mpi::communicator comm)
 {
   const auto js = stream->get_json();
@@ -676,6 +712,9 @@ static inline nlohmann::json args_to_input_stream_json(cxxopts::ParseResult& res
       j["filenames"] = results["input"].as<std::string>();
   }
 
+  if (results.count("async"))
+    j["async"] = true;
+
   if (results.count("synthetic")) {
     j["type"] = "synthetic";
     j["name"] = results["synthetic"].as<std::string>();
@@ -684,7 +723,8 @@ static inline nlohmann::json args_to_input_stream_json(cxxopts::ParseResult& res
 
   if (results.count("input-format")) j["format"] = results["input-format"].as<std::string>();
   if (results.count("dim")) j["nd"] = results["dim"].as<std::string>();
-  
+  if (results.count("bounds")) j["bounds"] = results["bounds"].as<std::string>();
+
   std::vector<size_t> dims;
   if (results.count("depth")) {
     dims.resize(3);
@@ -733,6 +773,7 @@ int parse_arguments(int argc, char **argv, diy::mpi::communicator comm)
     ("w,width", "Width", cxxopts::value<size_t>())
     ("h,height", "Height", cxxopts::value<size_t>())
     ("d,depth", "Depth (valid only for 3D regular grid data)", cxxopts::value<size_t>())
+    ("bounds", "Bounds for resampling to regular grid data", cxxopts::value<std::string>())
     ("n,timesteps", "Number of timesteps", cxxopts::value<size_t>(ntimesteps))
     ("start-timestep", "Start timestep", cxxopts::value<size_t>(start_timestep))
     ("var", "Variable name(s), e.g. `scalar', `u,v,w'.  Valid only for NetCDF, HDF5, and VTK.", cxxopts::value<std::string>())
@@ -795,6 +836,8 @@ int parse_arguments(int argc, char **argv, diy::mpi::communicator comm)
      cxxopts::value<std::string>(post_processing_options)->default_value(""))
     ("no-robust-detection", "Disable robust detection (faster than robust detection)",
      cxxopts::value<bool>(disable_robust_detection))
+    ("async", "Asynchronous I/O", 
+     cxxopts::value<bool>(async))
     ("v,verbose", "Verbose outputs", cxxopts::value<bool>(verbose))
     ("help", "Print usage", cxxopts::value<bool>(help));
   auto results = options.parse(argc, argv);
@@ -832,6 +875,8 @@ int parse_arguments(int argc, char **argv, diy::mpi::communicator comm)
     initialize_contour_tracker(comm); 
   else if (ttype == TRACKER_TDGL_VORTEX)
     initialize_tdgl_tracker(comm);
+  else if (ttype == TRACKER_PARTICLE)
+    initialize_particle_tracer(comm);
   else if (ttype == TRACKER_XGC_BLOB_FILAMENT)
     initialize_xgc_blob_filament_tracker(comm);
   else if (ttype == TRACKER_XGC_BLOB_THRESHOLD)
