@@ -1,7 +1,6 @@
 #define CATCH_CONFIG_RUNNER
 #include "catch.hh"
 #include "constants.hh"
-#include <ftk/filters/critical_point_tracker_wrapper.hh>
 #include <ftk/filters/critical_point_tracker_2d_unstructured.hh>
 
 using nlohmann::json;
@@ -18,7 +17,7 @@ TEST_CASE("critical_point_tracking_woven_unstructured") {
   m.from_vtk_unstructured_grid_file(mesh_filename);
   m.build_smoothing_kernel(kernel_size);
 
-  ftk::critical_point_tracker_2d_unstructured tracker(m);
+  ftk::critical_point_tracker_2d_unstructured tracker(world, m);
   tracker.initialize();
 
   for (int i = 0; i < nt; i ++) {
@@ -28,7 +27,7 @@ TEST_CASE("critical_point_tracking_woven_unstructured") {
     );
 
     ftk::ndarray<double> scalar, grad, J;
-    m.smooth_scalar_gradient_jacobian(data, kernel_size, scalar, grad, J);
+    m.smooth_scalar_gradient_jacobian(data, scalar, grad, J);
     scalar.reshape({1, scalar.dim(0)});
 
     tracker.push_field_data_snapshot(scalar, grad, J);
@@ -47,10 +46,4 @@ TEST_CASE("critical_point_tracking_woven_unstructured") {
 }
 #endif
 
-int main(int argc, char **argv)
-{
-  diy::mpi::environment env;
-  
-  Catch::Session session;
-  return session.run(argc, argv);
-}
+#include "main.hh"
