@@ -42,8 +42,4 @@ TEST_CASE("solve_cubic_real")
   }
 }
 
-int main(int argc, char **argv)
-{
-  Catch::Session session;
-  return session.run(argc, argv);
-}
+#include "main.hh"
